@@ -14,19 +14,20 @@
 
 """Internal YAML helpers."""
 
-import yaml
+from typing import Any, Optional, TextIO, Union
 
+import yaml
 
 # Use C speedups if available
 _safe_loader = getattr(yaml, 'CSafeLoader', yaml.SafeLoader)
 _safe_dumper = getattr(yaml, 'CSafeDumper', yaml.SafeDumper)
 
 
-def safe_load(stream):
+def safe_load(stream: Union[str, TextIO]) -> Any:
     """Same as yaml.safe_load, but use fast C loader if available."""
-    return yaml.load(stream, Loader=_safe_loader)
+    return yaml.load(stream, Loader=_safe_loader)  # type: ignore
 
 
-def safe_dump(data, stream=None, **kwargs):
+def safe_dump(data: Any, stream: Optional[TextIO] = None) -> str:
     """Same as yaml.safe_dump, but use fast C dumper if available."""
-    return yaml.dump(data, stream=stream, Dumper=_safe_dumper, **kwargs)
+    return yaml.dump(data, stream=stream, Dumper=_safe_dumper)  # type: ignore
