@@ -14,32 +14,35 @@
 
 """Main entry point to the Operator Framework."""
 
-import inspect
 import logging
 import os
 import shutil
 import subprocess
 import sys
-import typing
 import warnings
 from pathlib import Path
-
-import yaml
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Type, Union, cast
 
 import ops.charm
 import ops.framework
 import ops.model
 import ops.storage
+from ops.charm import CharmMeta
 from ops.jujuversion import JujuVersion
 from ops.log import setup_root_logging
 
+if TYPE_CHECKING:
+    from ops.charm import CharmBase
+    from ops.framework import BoundEvent, EventSource
+    from ops.model import Relation
+
 CHARM_STATE_FILE = '.unit-state.db'
 
 
 logger = logging.getLogger()
 
 
-def _exe_path(path: Path) -> typing.Optional[Path]:
+def _exe_path(path: Path) -> Optional[Path]:
     """Find and return the full path to the given binary.
 
     Here path is the absolute path to a binary, but might be missing an extension.
@@ -54,13 +57,14 @@ def _get_charm_dir():
     charm_dir = os.environ.get("JUJU_CHARM_DIR")
     if charm_dir is None:
         # Assume $JUJU_CHARM_DIR/lib/op/main.py structure.
-        charm_dir = Path('{}/../../..'.format(__file__)).resolve()
+        charm_dir = Path(f'{__file__}/../../..').resolve()
     else:
         charm_dir = Path(charm_dir).resolve()
     return charm_dir
 
 
-def _create_event_link(charm, bound_event, link_to):
+def _create_event_link(charm: 'CharmBase', bound_event: 'EventSource[Any]',
+                       link_to: Union[str, Path]):
     """Create a symlink for a particular event.
 
     Args:
@@ -68,19 +72,22 @@ def _create_event_link(charm, bound_event, link_to):
         bound_event: An event for which to create a symlink.
         link_to: What the event link should point to
     """
+    # type guard
+    assert bound_event.event_kind, f"unbound EventSource {bound_event}"
+
     if issubclass(bound_event.event_type, ops.charm.HookEvent):
         event_dir = charm.framework.charm_dir / 'hooks'
         event_path = event_dir / bound_event.event_kind.replace('_', '-')
     elif issubclass(bound_event.event_type, ops.charm.ActionEvent):
         if not bound_event.event_kind.endswith("_action"):
             raise RuntimeError(
-                'action event name {} needs _action suffix'.format(bound_event.event_kind))
+                f'action event name {bound_event.event_kind} needs _action suffix')
         event_dir = charm.framework.charm_dir / 'actions'
         # The event_kind is suffixed with "_action" while the executable is not.
         event_path = event_dir / bound_event.event_kind[:-len('_action')].replace('_', '-')
     else:
         raise RuntimeError(
-            'cannot create a symlink: unsupported event type {}'.format(bound_event.event_type))
+            f'cannot create a symlink: unsupported event type {bound_event.event_type}')
 
     event_dir.mkdir(exist_ok=True)
     if not event_path.exists():
@@ -94,7 +101,7 @@ def _create_event_link(charm, bound_event, link_to):
         event_path.symlink_to(target_path)
 
 
-def _setup_event_links(charm_dir, charm):
+def _setup_event_links(charm_dir: Path, charm: 'CharmBase'):
     """Set up links for supported events that originate from Juju.
 
     Whether a charm can handle an event or not can be determined by
@@ -109,11 +116,6 @@ def _setup_event_links(charm_dir, charm):
         charm: An instance of the Charm class.
 
     """
-    # XXX: on windows this function does not accomplish what it wants to:
-    #      it creates symlinks with no extension pointing to a .py
-    #      and juju only knows how to handle .exe, .bat, .cmd, and .ps1
-    #      so it does its job, but does not accomplish anything as the
-    #      hooks aren't 'callable'.
     link_to = os.path.realpath(os.environ.get("JUJU_DISPATCH_PATH", sys.argv[0]))
     for bound_event in charm.on.events().values():
         # Only events that originate from Juju need symlinks.
@@ -121,7 +123,7 @@ def _setup_event_links(charm_dir, charm):
             _create_event_link(charm, bound_event, link_to)
 
 
-def _emit_charm_event(charm, event_name):
+def _emit_charm_event(charm: 'CharmBase', event_name: str):
     """Emits a charm event based on a Juju event name.
 
     Args:
@@ -142,14 +144,24 @@ def _emit_charm_event(charm, event_name):
         event_to_emit.emit(*args, **kwargs)
 
 
-def _get_event_args(charm, bound_event):
+def _get_event_args(charm: 'CharmBase',
+                    bound_event: 'BoundEvent[Any]') -> Tuple[List[Any], Dict[str, Any]]:
     event_type = bound_event.event_type
     model = charm.framework.model
 
+    relation = None
     if issubclass(event_type, ops.charm.WorkloadEvent):
         workload_name = os.environ['JUJU_WORKLOAD_NAME']
         container = model.unit.get_container(workload_name)
         return [container], {}
+    elif issubclass(event_type, ops.charm.SecretEvent):
+        args: List[Any] = [
+            os.environ['JUJU_SECRET_ID'],
+            os.environ.get('JUJU_SECRET_LABEL'),
+        ]
+        if issubclass(event_type, (ops.charm.SecretRemoveEvent, ops.charm.SecretExpiredEvent)):
+            args.append(int(os.environ['JUJU_SECRET_REVISION']))
+        return args, {}
     elif issubclass(event_type, ops.charm.StorageEvent):
         storage_id = os.environ.get("JUJU_STORAGE_ID", "")
         if storage_id:
@@ -160,35 +172,40 @@ def _get_event_args(charm, bound_event):
             storage_name = "-".join(bound_event.event_kind.split("_")[:-2])
 
         storages = model.storages[storage_name]
-        id, storage_location = model._backend._storage_event_details()
+        index, storage_location = model._backend._storage_event_details()
         if len(storages) == 1:
             storage = storages[0]
         else:
             # If there's more than one value, pick the right one. We'll realize the key on lookup
-            storage = next((s for s in storages if s.id == id), None)
-        storage.location = storage_location
+            storage = next((s for s in storages if s.index == index), None)
+        storage = cast(Union[ops.storage.JujuStorage, ops.storage.SQLiteStorage], storage)
+        storage.location = storage_location  # type: ignore
         return [storage], {}
     elif issubclass(event_type, ops.charm.RelationEvent):
         relation_name = os.environ['JUJU_RELATION']
         relation_id = int(os.environ['JUJU_RELATION_ID'].split(':')[-1])
-        relation = model.get_relation(relation_name, relation_id)
-    else:
-        relation = None
+        relation: Optional[Relation] = model.get_relation(relation_name, relation_id)
 
     remote_app_name = os.environ.get('JUJU_REMOTE_APP', '')
     remote_unit_name = os.environ.get('JUJU_REMOTE_UNIT', '')
-    if remote_app_name or remote_unit_name:
-        if not remote_app_name:
-            if '/' not in remote_unit_name:
-                raise RuntimeError('invalid remote unit name: {}'.format(remote_unit_name))
-            remote_app_name = remote_unit_name.split('/')[0]
-        args = [relation, model.get_app(remote_app_name)]
-        if remote_unit_name:
-            args.append(model.get_unit(remote_unit_name))
-        return args, {}
-    elif relation:
-        return [relation], {}
-    return [], {}
+    departing_unit_name = os.environ.get('JUJU_DEPARTING_UNIT', '')
+
+    if not remote_app_name and remote_unit_name:
+        if '/' not in remote_unit_name:
+            raise RuntimeError(f'invalid remote unit name: {remote_unit_name}')
+        remote_app_name = remote_unit_name.split('/')[0]
+
+    kwargs: Dict[str, Any] = {}
+    if remote_app_name:
+        kwargs['app'] = model.get_app(remote_app_name)
+    if remote_unit_name:
+        kwargs['unit'] = model.get_unit(remote_unit_name)
+    if departing_unit_name:
+        kwargs['departing_unit_name'] = departing_unit_name
+
+    if relation:
+        return [relation], kwargs
+    return [], kwargs
 
 
 class _Dispatcher:
@@ -217,7 +234,7 @@ class _Dispatcher:
         else:
             self._init_legacy()
 
-    def ensure_event_links(self, charm):
+    def ensure_event_links(self, charm: 'CharmBase'):
         """Make sure necessary symlinks are present on disk."""
         if self.is_dispatch_aware:
             # links aren't needed
@@ -246,7 +263,6 @@ class _Dispatcher:
 
         dispatch_path = _exe_path(self._charm_dir / self._dispatch_path)
         if dispatch_path is None:
-            logger.debug("Legacy %s does not exist.", self._dispatch_path)
             return
 
         # super strange that there isn't an is_executable
@@ -276,7 +292,7 @@ class _Dispatcher:
         """Sets the name attribute to that which can be inferred from the given path."""
         name = path.name.replace('-', '_')
         if path.parent.name == 'actions':
-            name = '{}_action'.format(name)
+            name = f'{name}_action'
         self.event_name = name
 
     def _init_legacy(self):
@@ -317,16 +333,16 @@ class _Dispatcher:
         return self.event_name in ('collect_metrics',)
 
 
-def _should_use_controller_storage(db_path: Path, meta: ops.charm.CharmMeta) -> bool:
+def _should_use_controller_storage(db_path: Path, meta: CharmMeta) -> bool:
     """Figure out whether we want to use controller storage or not."""
     # if you've previously used local state, carry on using that
     if db_path.exists():
-        logger.debug("Using local storage: %s already exists", db_path)
         return False
 
-    # if you're not in k8s you don't need controller storage
-    if 'kubernetes' not in meta.series:
-        logger.debug("Using local storage: not a kubernetes charm")
+    # only use controller storage for Kubernetes podspec charms
+    is_podspec = 'kubernetes' in meta.series
+    if not is_podspec:
+        logger.debug("Using local storage: not a Kubernetes podspec charm")
         return False
 
     # are we in a new enough Juju?
@@ -340,7 +356,8 @@ def _should_use_controller_storage(db_path: Path, meta: ops.charm.CharmMeta) ->
         return False
 
 
-def main(charm_class: typing.Type[ops.charm.CharmBase], use_juju_for_storage: bool = None):
+def main(charm_class: Type[ops.charm.CharmBase],
+         use_juju_for_storage: Optional[bool] = None):
     """Setup the charm and dispatch the observed event.
 
     The event name is based on the way this executable was called (argv[0]).
@@ -354,10 +371,10 @@ def main(charm_class: typing.Type[ops.charm.CharmBase], use_juju_for_storage: bo
     """
     charm_dir = _get_charm_dir()
 
-    model_backend = ops.model._ModelBackend()
+    model_backend = ops.model._ModelBackend()  # pyright: reportPrivateUsage=false
     debug = ('JUJU_DEBUG' in os.environ)
     setup_root_logging(model_backend, debug=debug)
-    logger.debug("Operator Framework %s up and running.", ops.__version__)
+    logger.debug("Operator Framework %s up and running.", ops.__version__)  # type:ignore
 
     dispatcher = _Dispatcher(charm_dir)
     dispatcher.run_any_legacy_hook()
@@ -369,9 +386,7 @@ def main(charm_class: typing.Type[ops.charm.CharmBase], use_juju_for_storage: bo
     else:
         actions_metadata = None
 
-    if not yaml.__with_libyaml__:
-        logger.debug('yaml does not have libyaml extensions, using slower pure Python yaml loader')
-    meta = ops.charm.CharmMeta.from_yaml(metadata, actions_metadata)
+    meta = CharmMeta.from_yaml(metadata, actions_metadata)
     model = ops.model.Model(meta, model_backend)
 
     charm_state_path = charm_dir / CHARM_STATE_FILE
@@ -383,11 +398,15 @@ def main(charm_class: typing.Type[ops.charm.CharmBase], use_juju_for_storage: bo
 
     if use_juju_for_storage is None:
         use_juju_for_storage = _should_use_controller_storage(charm_state_path, meta)
+    elif use_juju_for_storage:
+        warnings.warn("Controller storage is deprecated; it's intended for "
+                      "podspec charms and will be removed in a future release.",
+                      category=DeprecationWarning)
 
     if use_juju_for_storage:
         if dispatcher.is_restricted_context():
             # TODO: jam 2020-06-30 This unconditionally avoids running a collect metrics event
-            #  Though we eventually expect that juju will run collect-metrics in a
+            #  Though we eventually expect that Juju will run collect-metrics in a
             #  non-restricted context. Once we can determine that we are running collect-metrics
             #  in a non-restricted context, we should fire the event as normal.
             logger.debug('"%s" is not supported when using Juju for storage\n'
@@ -398,20 +417,11 @@ def main(charm_class: typing.Type[ops.charm.CharmBase], use_juju_for_storage: bo
         store = ops.storage.JujuStorage()
     else:
         store = ops.storage.SQLiteStorage(charm_state_path)
-    framework = ops.framework.Framework(store, charm_dir, meta, model)
+    framework = ops.framework.Framework(store, charm_dir, meta, model,
+                                        event_name=dispatcher.event_name)
     framework.set_breakpointhook()
     try:
-        sig = inspect.signature(charm_class)
-        try:
-            sig.bind(framework)
-        except TypeError:
-            msg = (
-                "the second argument, 'key', has been deprecated and will be "
-                "removed after the 0.7 release")
-            warnings.warn(msg, DeprecationWarning)
-            charm = charm_class(framework, None)
-        else:
-            charm = charm_class(framework)
+        charm = charm_class(framework)
         dispatcher.ensure_event_links(charm)
 
         # TODO: Remove the collect_metrics check below as soon as the relevant
